import elixir
from datetime import datetime

##########
# Model
##########

class Property(elixir.Entity):

  elixir.using_options(tablename='properties')

  #NOTE Using the name as primary key would make the properties_values clearer,
  # but increases the db size (and may slow down joins)
  #name = elixir.Field(elixir.Text, primary_key=True)

  name = elixir.Field(elixir.Text)
  useParamDB = elixir.Field(elixir.Boolean)

  # TODO incorporate ParamDB here

class PropertyValue(elixir.Entity):

  elixir.using_options(tablename='properties_values')

  property = elixir.ManyToOne('Property')
  value = elixir.Field(elixir.Text) # value as a string
  hostname = elixir.Field(elixir.Text)
  set = elixir.Field(elixir.Text)
  created = elixir.Field(elixir.DateTime, default=datetime.now)
  lastUpdate = elixir.Field(elixir.DateTime, default=datetime.now)
  comment = elixir.Field(elixir.Text)
   
  def __repr__(self):
    return '<Property: Name: %s, Value: %s, Hostname: %s, Set: %s, ' \
        'Use ParamDB: %s, Comment: %s>' % \
        (self.property.name, self.value, self.hostname, self.set, \
         self.property.useParamDB, self.comment)

##########
# Helpers
##########

# returns list of property name, value tuples
def getPropertyNameValuesForGivenSetsAndHostnames(dburi, sets={}, hostnames={}):
  import sqlalchemy
  elixir.metadata.bind = sqlalchemy.create_engine(dburi, echo=False)
  elixir.setup_all()

  propertyNameValues = []
  for p in PropertyValue.query.all():
    propertyNameValues.append((p.property.name, p.value))

  return propertyNameValues

##########
# Test
##########

if __name__ == '__main__':
  import sqlalchemy
  engine = sqlalchemy.create_engine('sqlite:///test-configs/configuration1.s3db', 
      echo=False)
  elixir.metadata.bind = engine
  elixir.setup_all()
  elixir.create_all()

  p = Property(name='a.b', useParamDB = True)
  print PropertyValue(property=p, value='c')
  print PropertyValue(property=p, value='d')

  elixir.session.flush()  
