Ext.namespace("Paraxip",
              "Paraxip.status",
              "Paraxip.status.system",
              "Paraxip.status.controls",
              "Paraxip.status.channels");

Paraxip.pingGatewayRepeatCount = 20;

/**
 * Starts the autoRefresh of the errorGrid, the statsGrid and the channelGrid
 * with an interval equal to the value of the Paraxip.status.controls.autoRefreshTime
 * field. If the field is invalid (negative number, lower 5 seconds, empty field),
 * a message is displayed warning the user and the function returns before starting
 * the automatic refresh.
 */
Paraxip.status.controls.setAutomaticRefreshTime = function(){
  var timerField = Ext.getCmp('Paraxip.status.controls.autoRefreshTime');
  if( !timerField.isValid() ) {
    Ext.Msg.alert("Time invalid!","The time field is invalid. " +
                  "Please enter a positive integer equal or higher to 1 second.");
    return;
  }
  var value = timerField.getRawValue();
  var errorGrid = Paraxip.status.system.errorGrid;
  var statsGrid = Paraxip.status.system.statsGrid;
  var channelGrid = Paraxip.status.channels.channelGrid;
  if( errorGrid.rendered ) {
    errorGrid.getStore().startAutoRefresh(value,'','',true);
  }
  if( statsGrid.rendered ) {
    statsGrid.getStore().startAutoRefresh(value,'','',true);
  }
  if( channelGrid.rendered ) {
    channelGrid.getStore().startAutoRefresh(value,'','',true);
  }
};

/**
 * Stops the automatic refresh of the grids in Status&Controls. Might be useful
 * in development mode only.
 */
Paraxip.status.controls.stopAutomaticRefreshTime = function(){
  Paraxip.status.system.errorGrid.getStore().stopAutoRefresh();
  Paraxip.status.system.statsGrid.getStore().stopAutoRefresh();
  Paraxip.status.channels.channelGrid.getStore().stopAutoRefresh();
};

Paraxip.status.controls.startGwButton = new Ext.Button({
  id: 'Paraxip.status.controls.startGwButton',
  text: 'Start Gateway',
  disabled: true,
  minWidth: 95
});

Paraxip.status.controls.stopGwButton = new Ext.Button({
  id: 'Paraxip.status.controls.stopGwButton',
  text: 'Stop Gateway',
  disabled: true,
  minWidth: 95
});

Paraxip.status.controls.refreshGWStatus = new Ext.Button({
  id: 'Paraxip.status.controls.refreshGWStatus',
  text: 'Refresh Gateway Status',
  minWidth: 95,
  listeners: {
    'click':Paraxip.isGwStartedCall
  }
});

Paraxip.status.controls.startGwButton.on('click', function() {
  Paraxip.changeGwStatusBar(Paraxip.serviceStartingStatus, true, true);
  Paraxip.showProgressBar("Starting the gateway service...");
  Paraxip.disableItems(true);
  Ext.Ajax.request({
    url: 'status/startGatewayButton',
    disableCaching: true,
    //1 minute timeout to prevent error when gateway takes a long time to start
    timeout: 60000, 
    params: {js:Ext.util.JSON.encode('dummy param')},
    success: function(result){
      var response = Ext.util.JSON.decode(result.responseText);
      if (response.value) {
        Paraxip.serviceStarted = true;
        Paraxip.changeGwStatusBar(Paraxip.initializingApplicationStatus, true, true);
        Paraxip.updateProgressBarText("Service started. Initializing Gateway Application...");
        Paraxip.pingGatewayRepeatCount = 20;
        Ext.TaskMgr.start(Paraxip.status.controls.pingGwTask);
      } else {
        Paraxip.hideProgressBar();
        Paraxip.changeGwStatusBar(Paraxip.downStatus, false, true);
        Ext.Msg.alert("Unable to start the gateway","The gateway failed " +
                     "to start. Please be sure there is a valid configuration " +
                     "file before launching the gateway. This can be " +
                     "accomplished by running the Quick Setup.");
      }
    },
    failure: function(){
      Paraxip.hideProgressBar();
      Paraxip.changeGwStatusBar(Paraxip.downStatus, false, true);
      Paraxip.failureMessage();
    }
  });
});

Paraxip.status.controls.pingGwTask = {
  run: function(){Paraxip.status.controls.isApplicationStarted();},
  interval: 2000
};

Paraxip.status.controls.isApplicationStarted = function(){
  Paraxip.pingGatewayRepeatCount = Paraxip.pingGatewayRepeatCount - 1 ;
  if( Paraxip.pingGatewayRepeatCount == 0 ) {
    Ext.TaskMgr.stop(Paraxip.status.controls.pingGwTask);
    Paraxip.hideProgressBar();
    Paraxip.changeGwStatusBar(Paraxip.downStatus, false, true);
    Ext.Msg.alert("Unable to start the gateway","The gateway failed " +
                 "to start. Please be sure there is a valid configuration " +
                 "file before launching the gateway. This can be " +
                 "accomplished by running the Quick Setup.");
    return;
  }
  Ext.Ajax.request({
    url: 'status/pingGateway',
    disableCaching: true,
    timeout: 60000, 
    params: {js:Ext.util.JSON.encode('dummy param')},
    success: function(result){
      var response = Ext.util.JSON.decode(result.responseText);
      if (response.value) {
        Ext.TaskMgr.stop(Paraxip.status.controls.pingGwTask);
        Paraxip.isConfigOutOfSynch(false);
        Paraxip.isGwStartedCall();
        Paraxip.hideProgressBar();
      } 
    },
    failure: function(){
      Ext.TaskMgr.stop(Paraxip.status.controls.pingGwTask);
      Paraxip.hideProgressBar();
      Paraxip.changeGwStatusBar(Paraxip.downStatus, false, true);
      Ext.Msg.alert("Unable to start the gateway","The gateway failed " +
                   "to start. Please be sure there is a valid configuration " +
                   "file before launching the gateway. This can be " +
                   "accomplished by running the Quick Setup.");
    }
  });
};

Paraxip.status.controls.stopGwButtonMethod = function(answer) {
  if ( answer == "yes" ) {
    Paraxip.serviceStarted = false;
    Paraxip.changeGwStatusBar(Paraxip.stoppingStatus, true, true);
    Paraxip.showProgressBar("Stopping the gateway...");
    Paraxip.disableItems(true);
    Ext.Ajax.request({
      url: 'status/stopGatewayButton',
      disableCaching: true,
      params: {js:Ext.util.JSON.encode('dummy param')},
      success: function(result){
        var response = Ext.util.JSON.decode(result.responseText);
        if (response.value) {
          Paraxip.isGwStartedCall();
        } else {
          Paraxip.changeGwStatusBar(Paraxip.downStatus, false, true);
          Ext.Msg.alert("Unable to stop the gateway","Failed to stop the gateway. " +
                        "Gateway may already be stopped.");
        }
        Paraxip.hideProgressBar();
      },
      failure: function(){
        Paraxip.hideProgressBar();
        Paraxip.changeGwStatusBar(Paraxip.downStatus, false, true);
        Paraxip.failureMessage();
      }
    });
  }
};

Paraxip.status.controls.stopGwButton.on('click', function(){
  Ext.Msg.confirm("Stopping Gateway!","You are about to stop the gateway." + 
                  "Are you sure you want to proceed?",
                  Paraxip.status.controls.stopGwButtonMethod);
});

Paraxip.status.controls.autoRefreshTime = new Ext.form.NumberField({
  id: 'Paraxip.status.controls.autoRefreshTime',
  allowNegative: false,
  allowDecimals: false,
  allowBlank: false,
  value: 30,
  minValue: 1,
  anchor: '50%',
  fieldLabel: 'Automatic refresh interval (in s). Minimum value is 1 second.',
  listeners: {
    'render': function(){
      var elp = this.el.findParent('.x-form-element', 5, true);
      this.helpImage = elp.createChild({
        style : 'width:16px;height:18px;position:relative;left:30px;top:-23px;display:block;background:transparent url(/static/images/help.gif) no-repeat 0 2px;cursor:pointer!important;'
      });

      this.helpImage.alignTo(this.el, 'tl-tr', [2, 0]);

      this.helpImage.on("click", function()
      {
        helpWindow.setTitle("Automatic Refresh Help");
        helpWindow.show();

        var textarea = Ext.getDom("helpText_area");
        textarea.value = "This field represents the time interval in wich the grids located in the" +
                      "System Status and Channel Status tabs will automatically refresh their data." +
                      "To change the value, enter a positive integer equal or higher to one second" +
                      " and click on the Set Interval button.";
      }, this);//scope
    }
  }
});

Paraxip.status.controls.setAutoStartGW = function(){
  var value = this.getValue();
  Ext.Ajax.request({
    url: 'status/changeGWStartupType',
    method: 'POST',
    disableCaching: true,
    params: {
      'checkBoxValue':Ext.util.JSON.encode(value)
    }
  });
};

Paraxip.status.controls.statusButtonsPanel = new Ext.form.FormPanel({
  id        : 'Paraxip.status.controls.statusButtonsPanel',
  title     : 'Status options',
  colspan   : 1,
  width     : 486,
  height    : 200,
  labelWidth: 200,
  bodyStyle : "padding: 10px;",
  items     : [
    Paraxip.status.controls.autoRefreshTime,
  {
    xtype: 'button',
    id: 'Paraxip.status.controls.setRefreshTime',
    text: 'Set Interval',
    minWidth: 100,
    listeners: {
      'click':Paraxip.status.controls.setAutomaticRefreshTime,
      'disable':Paraxip.status.controls.stopAutomaticRefreshTime
    }
  },
  {
    xtype: 'label',
    html: '<html><br/></html>'
  },
    Paraxip.status.controls.refreshGWStatus
  ]
});

Paraxip.status.controls.autoStartGW = new Ext.form.Checkbox({
  id          : 'Paraxip.status.controls.autoStartGW',
  fieldLabel  : "Automatically start gateway at startup",
  width       : 30,
  disabled    : true,
  listeners   : {
    'check':Paraxip.status.controls.setAutoStartGW,
    'render':function(){
      if( !isGatewayInCLI ) {
        this.enable();
      }
      Ext.Ajax.request({
        url:'status/isGWAutomaticStartup',
        success: function(result){
          var response = Ext.util.JSON.decode(result.responseText);
          Paraxip.status.controls.autoStartGW.setValue(response.value);
        }
      });
    }
  }
});

Paraxip.status.controls.controlsButtonsPanel = new Ext.form.FormPanel({
  id        : 'Paraxip.status.controls.controlsButtonsPanel',
  title     : 'Gateway Controls',
  colspan   : 1,
  width     : 486,
  height    : 200,
  bodyStyle : "padding: 10px;",
  items     : [
    Paraxip.status.controls.startGwButton,
    {
      xtype: 'label',
      html: '<html><br/></html>'
    },
    Paraxip.status.controls.stopGwButton,
    {
      xtype: 'label',
      html: '<html><br/></html>'
    },
    Paraxip.status.controls.autoStartGW
  ]
});

Paraxip.status.controls.messagePanel = new Ext.Panel({
  id      : 'Paraxip.status.controls.messagePanel',
  colspan : 2,
  width   : 972,
  height  : 286,
  autoLoad: {url:'/static/gateway_down.html', nocache: false}
});

Paraxip.status.controls.controlsPanel = new Ext.Panel({
  id            : 'Paraxip.status.controls.controlsPanel',
  layout        : 'table',
  width         : 971,
  height        : 486,
  border        : false,
  layoutConfig  : {
    columns: 2
  },
  items         : [
    Paraxip.status.controls.controlsButtonsPanel,
    Paraxip.status.controls.statusButtonsPanel,
    Paraxip.status.controls.messagePanel
  ]
});


